<?php

$room_warnings = array();//here we store room related warnings
$wall_warnings = array();//here we store wall related warnings
$material_record = array();//we record all materials used in the room

/**
 * This filter sets up the rendering for VR
 */

add_filter( 'the_content', 'bimba_3d_ambient_render_in_the_content_filter' );

function bimba_3d_ambient_render_in_the_content_filter( $content ){
	
	if ( get_post_type() == 'bimba_3d_ambient' AND ! post_password_required() ){//it works only for bimba_3d_ambient cpt AND post_password_required()
		//starts rendering scene and assets
		$content .= '<a-scene style="width: 100%; height: 500px" embedded>
				<a-assets>';
		
		$id = get_the_ID();//for retreiving post meta
		
		$room_name = basename( get_permalink( $id ) );//for naming ids
		
		$room_set = get_post_meta( $id, '_3d_ambient_general_settings', true );//all the room general settings
		
		$walls = get_post_meta( $id, '_3d_ambient_group_walls', true );//all the wall general settings
		
		$content .= bimba_3d_ambient_register_assets( $id, $room_name, $walls, $room_set );
		
		$args = array(
			'post_parent' => $id,
			'post_type'   => 'bimba_3d_ambient', 
			'numberposts' => -1,
		);
		
		$children = get_children( $args );//get eventual children
		
		if ( $children ) {
			foreach ($children as $child ){
				
				$child_room_name = basename( get_permalink( $child->ID ) );//for naming ids
				
				$child_room_set = get_post_meta( $child->ID, '_3d_ambient_general_settings', true );//all the child room general settings
				
				$child_walls = get_post_meta( $child->ID, '_3d_ambient_group_walls', true );//all the child wall general settings
				
				$content .= bimba_3d_ambient_register_assets( $child->ID, $child_room_name, $child_walls , $child_room_set );
				
			}
		}
		
		global $material_record;
		foreach ($material_record as $key => $material_id) {
			$material_render = get_post_meta( $material_id, '_3d_material_render', true );
			$content .= '
				<img id="bimba-material-' . $material_id . '" src="' . $material_render [0] [ 'image' ] . '">';
			if ($material_render [0] [ 'frame_image' ]) {
				$content .= '
				<img id="bimba-frame-material-' . $material_id . '" src="' . $material_render [0] [ 'frame_image' ] . '">';
			}
		}
		
		$content .= '
				<img id="bimba-architect" src="' . plugins_url( 'images/architect.png', dirname( __FILE__ ) ) . '">';
		
		$content .= '
				</a-assets>';//here end the assets
		
		$coords = bimba_3d_ambient_extract_polygon_coords( $walls, $id );
		
		$centers = bimba_3d_ambient_extract_wall_centers( $coords );
		
		$floor_area = bimba_3d_ambient_extract_floor_area( $coords );
		
		$floor_dim = bimba_3d_ambient_extract_floor_dimensions( $centers );
		
		$content .= bimba_3d_ambient_render_room( $id, $room_name, $floor_area, $floor_dim, $walls, $centers, '0', $parent_coord , $room_set );
		
		if ( $children ) {
			//gets the parent global coordinates
			$parent_diff = get_post_meta( $id, '_3d_ambient_global_diff', true );
			$parent_coord = array(
				'parent_x'	=> $parent_diff [0] ['x'],
				'parent_y'	=> $parent_diff [0] ['y'],
				'parent_z'	=> $parent_diff [0] ['z'],
				'parent_rot'	=> $parent_diff [0] ['rot'],
			);
			
			foreach ($children as $child ){
		
				$child_room_name = basename( get_permalink( $child->ID ) );//for naming ids
				
				$child_room_set = get_post_meta( $child->ID, '_3d_ambient_general_settings', true );//all the child room general settings
		
				$child_walls = get_post_meta( $child->ID, '_3d_ambient_group_walls', true );
		
				$child_coords = bimba_3d_ambient_extract_polygon_coords( $child_walls, $child->ID );
				
				$child_centers = bimba_3d_ambient_extract_wall_centers( $child_coords );
				
				$child_floor_area = bimba_3d_ambient_extract_floor_area( $child_coords );
				
				$child_floor_dim = bimba_3d_ambient_extract_floor_dimensions( $child_centers );
				
				$content .= bimba_3d_ambient_render_room( $child->ID, $child_room_name, $child_floor_area, $child_floor_dim, $child_walls, $child_centers, '1', $parent_coord , $child_room_set );
		
			}
		}
		//entities are all set up, let's add background and camera
		$content .= '<a-sky color="black"></a-sky>
					<a-entity id="bimba-camera-ent">
        				<a-camera id="bimba-camera" wasd-controls="fly: ' . $room_set [0] ['camera_fly'] . '">
							<a-light type="point"></a-light>    
        					<a-entity position="0 -1.6 0" id="camera-foot"></a-entity>
							<a-cursor color="#2E3A87"></a-cursor>
        				</a-camera>
      				</a-entity>
				</a-scene>';//close a-scene tag 
		
		//here we display room data...
		
		$content .= bimba_3d_ambient_display_data( $id );
		
		//...and eventual child data
		if ( $children ) {
			$total_data = get_post_meta( $id, '_3d_ambient_data', true );
			foreach ($children as $child ){
				
				$content .= bimba_3d_ambient_display_data( $child->ID );
				
				$total_data = bimba_sum_arrays($total_data, get_post_meta( $child->ID, '_3d_ambient_data', true ));
			}
			$content .= bimba_3d_ambient_display_total_data( $total_data );
		}
		
	}
	
	return $content;
}

/**
 * Returns the polyline coordinates from wall info
 */

function bimba_3d_ambient_extract_polygon_coords( $walls, $id ){
	
	global $room_warnings;
	global $wall_warnings;
	
	$i = 1;
	foreach ($walls as $wall){
		
		switch ( $wall['direction'] ){
			case 0:
				$coords [$i]['x'] = $coords [$i-1]['x'] + $wall['length']/100;
				$coords [$i]['z'] = $coords [$i-1]['z'];
				break;
			case 1:
				$coords [$i]['x'] = $coords [$i-1]['x'];
				$coords [$i]['z'] = $coords [$i-1]['z'] + $wall['length']/100;
				break;
			case 2:
				$coords [$i]['x'] = $coords [$i-1]['x'] - $wall['length']/100;
				$coords [$i]['z'] = $coords [$i-1]['z'];
				break;
			case 3:
				$coords [$i]['x'] = $coords [$i-1]['x'];
				$coords [$i]['z'] = $coords [$i-1]['z'] - $wall['length']/100;
				break;
		}
		if ( ! $wall['length'] ) {
			$wall_warnings [ $id ][$i] = $i;
		}
		
		$i ++ ;
	}
	
	$room_warnings [ $id ]['x'] = $coords [$i-1]['x'] * 100;
	$room_warnings [ $id ]['z'] = $coords [$i-1]['z'] * 100;
	
	return $coords;
}

/**
 * Returns wall centers from polyline coordinates
 */

function bimba_3d_ambient_extract_wall_centers( $coords ){
	
	$centers = array();
	$i = 0;
		foreach ($coords as $coord){
			$centers [$i]['x'] = ( $coords [$i+1]['x'] - $coords [$i]['x'] ) / 2 + $coords [$i]['x'];
			$centers [$i]['z'] = ( $coords [$i+1]['z'] - $coords [$i]['z'] ) / 2 + $coords [$i]['z'];
			$i ++ ;
		}
		
	return $centers;
}

/**
 * Returns floor area
 */

function bimba_3d_ambient_extract_floor_area( $coords ){
	$i = 1;
	foreach ($coords as $coord){
		$tot1 = $tot1 + $coords [$i]['x'] * $coords [$i+1]['z'];
		$tot2 = $tot2 + $coords [$i+1]['x'] * $coords [$i]['z'];
		$i ++ ;
	}
	
	$floor_area = ( $tot1 - $tot2 ) / 2;
	return $floor_area;
}

/**
 * Returns the envelope of the wall polyline
 */

function bimba_3d_ambient_extract_floor_dimensions( $centers ){
	$floor_dim = array();
	foreach ($centers as $center){
		if( $xmin > $center ['x'] ){
			$xmin = $center ['x'];
		}
		if( $xmax < $center ['x'] ){
			$xmax = $center ['x'];
		}
		if( $zmin > $center ['z'] ){
			$zmin = $center ['z'];
		}
		if( $zmax < $center ['z'] ){
			$zmax = $center ['z'];
		}
	}
	$floor_dim ['width'] = $xmax - $xmin ;
	$floor_dim ['height'] = $zmax - $zmin ;
	
	return $floor_dim;
}

/**
 * Starts the a-scene tag and prepares the assets
 */

function bimba_3d_ambient_register_assets( $id, $room_name, $walls , $room_set ){//, $material_record
	
	global $material_record;
	//global images
	
	$floor_image = $room_set[ 0 ]['floor_image'] ;
	$ceiling_image = $room_set[ 0 ]['ceiling_image'] ;
	
	if ( $room_set[ 0 ]['floor_material'] ) {
		$material_record [$room_set[ 0 ]['floor_material']] = $room_set[ 0 ]['floor_material'];//records material ID as key AND value
	}
	if( $floor_image ){
		$return .= '
				<img id="' . $room_name . '-floor-image" src="' . $floor_image . '">';
	} 
	
	if ( $room_set[ 0 ]['wall_material'] ) {
		$material_record [$room_set[ 0 ]['wall_material']]= $room_set[ 0 ]['wall_material'];
	}
	
	if ( $room_set[ 0 ]['tiling_material'] ) {
		$material_record [$room_set[ 0 ]['tiling_material']]= $room_set[ 0 ]['tiling_material'];
	}
	
	if ( $room_set[ 0 ]['skirting_material'] ) {
		$material_record [$room_set[ 0 ]['skirting_material']]= $room_set[ 0 ]['skirting_material'];
	}
	
	if ( $room_set[ 0 ]['ceiling_material'] ) {
		$material_record [$room_set[ 0 ]['ceiling_material']]= $room_set[ 0 ]['ceiling_material'];
	}
	
	if( $ceiling_image ){
		$return .= '
				<img id="' . $room_name . '-ceiling-image" src="' . $ceiling_image . '">';
	}
	//local wall images
	$i = 0;
	foreach ($walls as $wall){
		$j = $i + 1;
		if( $wall [ 'image' ]){
			$return .= '
					<img id="' . $room_name . '-wall-image-' . $j . '" src="' . $wall [ 'image' ] . '">';
		}
		if( $wall [ 'wall_material' ]){
			$material_record [$wall [ 'wall_material' ]]= $wall [ 'wall_material' ];
		}
		if( $wall [ 'tiling_material' ]){
			$material_record [$wall [ 'tiling_material' ]]= $wall [ 'tiling_material' ];
		}
		if( $wall [ 'op_material' ]){
			$material_record [$wall [ 'op_material' ]]= $wall [ 'op_material' ];
		}
		$i ++;
	}
	
	return $return;
}

/**
 * Renders Room
 */

function bimba_3d_ambient_render_room( $id,  $room_name, $floor_area, $floor_dim, $walls, $centers, $global_flag, $parent_coord , $room_set ){
	
	//set all room variables
	
	$height = $room_set [0] ['height'] / 100;
	if ( ! $height ) {//if room has no height it's not worth rendering it!
		return $return;
	}
	
	$sk_height = $room_set [0] ['skirting_height'] / 100;
	
	$room_wall_render = bimba_3d_ambient_render_by_material( $room_set [0] ['wall_material'] );
	
	$room_floor_render = bimba_3d_ambient_render_by_material( $room_set [0] ['floor_material'] );
	
	$room_tiling_render = bimba_3d_ambient_render_by_material( $room_set [0] ['tiling_material'] );
	
	if ( $sk_height ){
		if ( $sk_height > 0.3 ){//we don't want skirtings higher than 30cm
			$sk_height = 0.3;
			$room_set [0] ['skirting_height'] = 30;
		}
		$room_skirting_render = bimba_3d_ambient_render_by_material( $room_set [0] ['skirting_material'] );
	}
	
	$room_ceiling_render = bimba_3d_ambient_render_by_material( $room_set [0] ['ceiling_material'] );
	
	//calculations first
	
	$paint_surf = $paint_surf + $floor_area;
	
	//Here we start rendering. The room is a whole entity, so you can move it around
	$return .= bimba_start_room_entity( $id, $room_name, $parent_coord, $global_flag);
	
	//next is for floor rendering
	
	if ( $room_set [0] ['floor_image'] ) {
		$floor_render = 'src: #' . $room_name . '-floor-image ; color: #ffffff';
		$return .= bimba_plane_entity_render_1_1( 'floor', $room_name, '0', $floor_dim ['width'], $floor_dim ['height'], 
				'0', '0', '0', '-90', '0', '0', $floor_render );
	} else {
		$return .= bimba_plane_entity_render( 'floor', $room_name, '0', $floor_dim ['width'], $floor_dim ['height'],
				'0', '0', '0', '-90', '0', '0', $room_floor_render );
	}
	
	//next is for ceiling rendering
	
	if ( $room_set [0] ['ceiling_image'] ) {
		$ceiling_render = 'src: #' . $room_name . '-ceiling-image ; color: #ffffff';
		$return .= bimba_plane_entity_render_1_1( 'ceiling', $room_name, '0', $floor_dim ['width'], $floor_dim ['height'],
				'0', $height, '0', '90', '0', '0', $ceiling_render );
	} else {
		$return .= bimba_plane_entity_render( 'ceiling', $room_name, '0', $floor_dim ['width'], $floor_dim ['height'],
				'0', $height, '0', '90', '0', '0', $room_ceiling_render );
	}
	
	//and the walls
	
	$objects = get_post_meta( $id, '_3d_ambient_group_objects', true );//we will need objects later
	
	$i = 0;
	foreach ($walls as $wall){
		
		//let's do the controls first
		
		$wall_length = $wall ['length'] / 100;//this is first and most important wall control
		if ( $wall_length ) {//we process the wall only if length is not null
		
			//here come secondary wall controls
			$tiling_height = $wall ['tiling_height'] / 100;
			if ( $tiling_height > $height ){//control if tiles are higher than room!
				$tiling_height = $height ;
				$walls [ $i ] [ 'tiling_height' ] = $tiling_height * 100;//TODO feedback changed data
			}
			if ( $wall ['openings'] ) {
				
				//retrieve data
				$op_width = $wall [ 'op_width' ] / 100;
				$op_height = $wall [ 'op_height' ] / 100;
				$op_offset = $wall [ 'op_offset' ] / 100;
				$wall_depth = $wall [ 'wall_depth' ] / 100;
				$sill_height = $wall [ 'sill_height' ] / 100;
				$sill_depth = $wall [ 'sill_depth' ] / 100;
				
				//controls
				if( $op_width > $wall_length ){
					$op_width = $wall_length;
				}
				if ( !$op_offset ) {
					$op_offset = ( $wall_length - $op_width ) / 2;
				}
				if( ( $op_offset + op_width ) > $wall_length ){
					$op_offset = $wall_length - op_width;
				}
				if ($op_offset < 0) {
					$op_offset = 0;
				}
				if( $op_height > $height ){
					$op_height = $height;
				}
				if ( $sill_height ) {
					if( $sill_height > $op_height ){
						$sill_height = $op_height;
						$wall_depth = $sill_depth;
					}
					if( $sill_depth > $wall_depth ){
						$sill_depth = $wall_depth;
					}
				} else {//there is no windowsill
					$sill_depth = 0;
				}
				
				$walls [ $i ] [ 'op_width' ] = $op_width * 100;//feedback eventual changed data
				$walls [ $i ] [ 'op_height' ] = $op_height * 100;
				//$walls [ $i ] [ 'op_offset' ] = $op_offset * 100;//can't feedback this one
				$walls [ $i ] [ 'wall_depth' ] = $wall_depth * 100;
				$walls [ $i ] [ 'sill_height' ] = $sill_height * 100;
				$walls [ $i ] [ 'sill_depth' ] = $sill_depth * 100;
			
			}
			
			//All the calculations
			
			switch ( $wall ['openings'] ){
				
				case 0://no opening
					if ( $tiling_height ){//do we have tiling?
						$tiling_surf = $tiling_surf + ( $wall_length * $tiling_height );
					} elseif ( $sk_height ){
						$sk_length = $sk_length + $wall_length;
					}
					$paint_surf = $paint_surf + ( $wall_length * ( $height - $tiling_height ) );
				break;
				
				case 1 OR 2://door or window
					//opening shoulders
					$tiling_surf = $tiling_surf + ( $wall_length - $op_width) * $tiling_height;
					$paint_surf = $paint_surf + ( $wall_length - $op_width) * ( $height - $tiling_height );
					//opening center
					$total_op_surf = $op_width * ( $height - $op_height ) + $op_width * $wall_depth + 2 * $wall_depth * $op_height -
					2 * $sill_depth * $sill_height + $op_width * $sill_depth +  $op_width * $sill_height;
					$tiling_op_surf = 0;
					if ( $tiling_height ){//do we have tiling?
						if ( $tiling_height > $op_height ){//tiling above window height
							$tiling_op_surf = $op_width * ( $tiling_height - $op_height ) + $op_width * $wall_depth + 2 * $wall_depth * $op_height -
							2 * $sill_depth * $sill_height + $op_width * $sill_depth +  $op_width * $sill_height;
						} elseif ( $tiling_height > $sill_height ){//tiling above windowsill height
							$tiling_op_surf = 2 * $wall_depth * $tiling_height -
							2 * $sill_depth * $sill_height + $op_width * $sill_depth +  $op_width * $sill_height;
						} else {//tiling under windowsill height
							$tiling_op_surf = 2 * ($wall_depth - $sill_depth) * $tiling_height + $op_width * $tiling_height;
						}
						$tiling_surf = $tiling_surf + $tiling_op_surf;
					} elseif ( $sk_height ){//skirting
						$sk_length = $sk_length + $wall_length - $op_width ;
						if ( $sill_height ){
							$sk_length = $sk_length + $op_width + ( $wall_depth - $sill_depth ) * 2;
						} else {
							$sk_length = $sk_length + ( $wall_depth * 2 ) ;
						}
					}
					$paint_surf = $paint_surf + $total_op_surf - $tiling_op_surf;
					
					//floor and framing
					$floor_area_op = $floor_area_op + $op_width * ( $wall_depth - $sill_depth );
					if ( $wall [ 'op_material' ] AND $op_width AND $op_height > $sill_height ){
						switch ( $wall ['openings'] ){
							case 1://door
								$door_number ++;
								$door_surf = $door_surf + $op_width * ( $op_height - $sill_height );
								break;
					
							case 2://window
								$window_number ++;
								$window_surf = $window_surf + $op_width * ( $op_height - $sill_height );
								break;
						}
					}
				break;
			}
			
			//let's start rendering!
			
			//entity that holds the wall element
			
			$j = $i + 1;
			$return .= '
					<a-entity id="' . $room_name . '-wall-ent-' . $j . '" 
					position="' . ( $centers [$i] ['x'] - $floor_dim ['width'] / 2 ) . ' 0 ' . ( $centers [$i] ['z'] - $floor_dim ['height'] / 2 ) . '" 
					rotation="0 ' . ( $wall ['direction'] * ( -90 ) ) . ' 0">';
			$return .= '
					<a-entity id="' . $room_name . '-wall-' . $j . '-label" scale="2 2 2" position="0 0.1 0.01" text="value: ' . esc_html__( 'Wall ' , 'bim-ba' ) . '-' . $j . ';color: black; align: center;"></a-entity>';
			
			//wall element 
			
			if ( $wall [ 'image' ] ) {
				$wall_render = 'src: #' . $room_name . '-wall-image-' . $j . '; color: #ffffff';
						$return .= bimba_plane_entity_render_1_1( 'wall', $room_name, $j, $wall_length, $height,
								'0', $height / 2, '0', '0', '0', '0', $wall_render );
			} else {
				
				//see if we have local colors, if not just get global colors
				$wall_material = get_post_meta( $wall ['wall_material'], '_3d_material_render', true );
				$wall_render = 'src: #bimba-material-' . $wall ['wall_material'] . '; color:' . $wall_material [0] ['color'];
				
				$tiling_material = get_post_meta( $wall ['tiling_material'], '_3d_material_render', true );
				$tiling_render = 'src: #bimba-material-' . $wall ['tiling_material'] . '; color:' . $tiling_material [0] ['color'];
				
				if( ! $wall ['wall_material'] ){
					$wall_render = $room_wall_render;
				}
				if( ! $wall ['tiling_material'] ){
					$tiling_render = $room_tiling_render;
				}
				
				switch ( $wall ['openings'] ){
					
					case 0://no opening
						
						$wall_height = $height;
						$wall_y = $height / 2;
						if ( $tiling_height ){
							$wall_height = $height - $tiling_height;
							$wall_y = ( $height - $tiling_height ) / 2 + $tiling_height;
							$return .= bimba_plane_entity_render( 'tiling', $room_name, $j, $wall_length, $tiling_height,
									'0', $tiling_height / 2, '0', '0', '0', '0', $tiling_render );
						} elseif ( $sk_height ) {
							$wall_height = $height - $sk_height;
							$wall_y = ( $height - $sk_height ) / 2 + $sk_height;
							$return .= bimba_plane_entity_render_1_1( 'skirting', $room_name, $j, $wall_length, $sk_height,
									'0', $sk_height / 2, '0', '0', '0', '0', $room_skirting_render );
						}
							
						$return .= bimba_plane_entity_render( 'wall', $room_name, $j, $wall_length, $wall_height,
								'0', $wall_y, '0', '0', '0', '0', $wall_render );
					break;
					
					case 1 OR 2://door or window
						//left shoulder
						if ( $op_offset ) {
							$wall_height = $height;
							$wall_y = $height / 2;
							if ( $tiling_height ) {
								$wall_height = $height - $tiling_height;
								$wall_y = ( $height - $tiling_height ) / 2 + $tiling_height;
								$return .= bimba_plane_entity_render( 'tiling', $room_name, ( $j . '-ls' ) , $op_offset, $tiling_height,
										- ( $wall_length - $op_offset ) / 2 , $tiling_height / 2 , '0', '0', '0', '0', $tiling_render );
							} elseif ( $sk_height ) {
								$wall_height = $height - $sk_height;
								$wall_y = ( $height - $sk_height ) / 2 + $sk_height;
								$return .= bimba_plane_entity_render_1_1( 'skirting', $room_name, ( $j . '-ls' ) , $op_offset, $sk_height,
										- ( $wall_length - $op_offset ) / 2 , $sk_height / 2 , '0', '0', '0', '0', $room_skirting_render );
							}
							$return .= bimba_plane_entity_render( 'wall', $room_name, ( $j . '-ls' ) , $op_offset, $wall_height,
									- ( $wall_length - $op_offset ) / 2 , $wall_y , '0', '0', '0', '0', $wall_render );
						}
						//right shoulder
						$op_offset_right = $wall_length - $op_offset - $op_width;
						if ( $op_offset_right ) {
							$wall_height = $height;
							$wall_y = $height / 2;
							if ( $tiling_height ) {
								$wall_height = $height - $tiling_height;
								$wall_y = ( $height - $tiling_height ) / 2 + $tiling_height;
								$return .= bimba_plane_entity_render( 'tiling', $room_name, ( $j . '-rs' ) , $op_offset_right, $tiling_height,
										( $wall_length - $op_offset_right ) / 2 , $tiling_height / 2 , '0', '0', '0', '0', $tiling_render );
									
							} elseif ( $sk_height ) {
								$wall_height = $height - $sk_height;
								$wall_y = ( $height - $sk_height ) / 2 + $sk_height;
								$return .= bimba_plane_entity_render_1_1( 'skirting', $room_name, ( $j . '-rs' ) , $op_offset_right, $sk_height,
										( $wall_length - $op_offset_right ) / 2 , $sk_height / 2 , '0', '0', '0', '0', $room_skirting_render );
							}
							$return .= bimba_plane_entity_render( 'wall', $room_name, ( $j . '-rs' ) , $op_offset_right, $wall_height,
									( $wall_length - $op_offset_right ) / 2 , $wall_y , '0', '0', '0', '0', $wall_render );
						}
						//lintel
						if ( $height - $op_height ) {
							$wall_height = $height - $op_height;
							$wall_y = ( $height - $op_height ) / 2 + $op_height;
							if ( $tiling_height > $op_height ) {
								$wall_height = $height - $tiling_height;
								$wall_y = ( $height - $tiling_height ) / 2 + $tiling_height;
								$return .= bimba_plane_entity_render( 'tiling', $room_name, ( $j . '-lint' ) , $op_width , $tiling_height - $op_height,
										( $op_offset + $op_width / 2 ) - $wall_length / 2 , ( $tiling_height - $op_height ) / 2 + $op_height , '0',
										'0', '0', '0', $tiling_render );
							}
							$return .= bimba_plane_entity_render( 'wall', $room_name, ( $j . '-lint' ) , $op_width , $wall_height,
									( $op_offset + $op_width / 2 ) - $wall_length / 2 , $wall_y , '0', '0', '0', '0', $wall_render );
						}
						//plafond
						if ( $wall_depth ) {
							if ( $tiling_height > $op_height ) {
								$return .= bimba_plane_entity_render( 'tiling', $room_name, ( $j . '-pla' ) , $op_width , $wall_depth,
										( $op_offset + $op_width / 2 ) - $wall_length / 2 , $op_height , - $wall_depth / 2, '90', '0', '0', $tiling_render );
							} else {
								$return .= bimba_plane_entity_render( 'wall', $room_name, ( $j . '-pla' ) , $op_width , $wall_depth,
										( $op_offset + $op_width / 2 ) - $wall_length / 2 , $op_height , - $wall_depth / 2, '90', '0', '0', $wall_render );
							}
						}
						//left depth
						if ( $wall_depth AND ! ( $wall['delete_side_wall'] == 1 OR $wall['delete_side_wall'] == 3 ) ) {
							if ( $sill_height ){
								//upper part
								$tiling_wall_height = $tiling_height;
								$wall_height = $op_height;
								$wall_y = ( $op_height - $sill_height ) / 2 + $sill_height;
								if ( $tiling_wall_height ) {
									if ( $tiling_wall_height > $wall_height ) {
										$tiling_wall_height = $wall_height;
									}
									if ( $tiling_wall_height > $sill_height ) {
										$wall_height = $op_height - $tiling_wall_height;
										$wall_y = ( $op_height - $tiling_wall_height ) / 2 + $tiling_wall_height;
										$return .= bimba_plane_entity_render( 'tiling', $room_name, ( $j . '-li' ) , $wall_depth, ( $tiling_wall_height - $sill_height ),
												- $wall_length / 2 + $op_offset , ( $tiling_wall_height - $sill_height ) / 2 + $sill_height, - $wall_depth / 2, '0', '90', '0', $tiling_render );
									} else {
										$wall_height = $op_height - $sill_height;
									}
								}
								$return .= bimba_plane_entity_render( 'wall', $room_name, ( $j . '-li' ) , $wall_depth , $wall_height ,
										- $wall_length / 2 + $op_offset, $wall_y, - $wall_depth / 2, '0', '90', '0', $wall_render );
								//lower part
								$tiling_wall_height = $tiling_height;
								$wall_height = $sill_height;
								$wall_y = $sill_height / 2;
								if ( $tiling_wall_height ) {
									if ( $tiling_height > $wall_height ) {
										$tiling_wall_height = $wall_height;
									}
									$wall_height = $sill_height - $tiling_wall_height;
									$wall_y = ( $sill_height - $tiling_wall_height ) / 2 + $tiling_wall_height;
									$return .= bimba_plane_entity_render( 'tiling', $room_name, ( $j . '-li2' ) , ( $wall_depth - $sill_depth ), $tiling_wall_height,
											- $wall_length / 2 + $op_offset , $tiling_wall_height / 2 , - ( $wall_depth - $sill_depth ) / 2, '0', '90', '0', $tiling_render );
								} elseif ( $sk_height ) {
									$wall_height = $sill_height - $sk_height;
									$wall_y = ( $sill_height - $sk_height ) / 2 + $sk_height;
									$return .= bimba_plane_entity_render_1_1( 'skirting', $room_name, ( $j . '-li2' ) , ( $wall_depth - $sill_depth ), $sk_height,
											- $wall_length / 2 + $op_offset , $sk_height / 2 , - ( $wall_depth - $sill_depth ) / 2, '0', '90', '0', $room_skirting_render );
								}
								$return .= bimba_plane_entity_render( 'wall', $room_name, ( $j . '-li2' ) , ( $wall_depth - $sill_depth ) , $wall_height ,
										- $wall_length / 2 + $op_offset, $wall_y, - ( $wall_depth - $sill_depth ) / 2, '0', '90', '0', $wall_render );
							} else {
								$tiling_wall_height = $tiling_height;
								$wall_height = $op_height;
								$wall_y = $op_height / 2;
								if ( $tiling_height ) {
									if ( $tiling_height > $wall_height ) {
										$tiling_wall_height = $wall_height;
									}
									$wall_height = $op_height - $tiling_wall_height;
									$wall_y = ( $op_height - $tiling_wall_height ) / 2 + $tiling_wall_height;
									$return .= bimba_plane_entity_render( 'tiling', $room_name, ( $j . '-li' ) , $wall_depth, $tiling_wall_height,
											- $wall_length / 2 + $op_offset , $tiling_wall_height / 2 , - $wall_depth / 2, '0', '90', '0', $tiling_render );
								} elseif ( $sk_height ) {
									$wall_height = $op_height - $sk_height;
									$wall_y = ( $op_height - $sk_height ) / 2 + $sk_height;
									$return .= bimba_plane_entity_render_1_1( 'skirting', $room_name, ( $j . '-li' ) , $wall_depth, $sk_height,
											- $wall_length / 2 + $op_offset , $sk_height / 2 , - $wall_depth / 2, '0', '90', '0', $room_skirting_render );
								}
								$return .= bimba_plane_entity_render( 'wall', $room_name, ( $j . '-li' ) , $wall_depth , $wall_height ,
										- $wall_length / 2 + $op_offset, $wall_y, - $wall_depth / 2, '0', '90', '0', $wall_render );
							}
							
						}
						//right depth
						if ( $wall_depth AND ! ( $wall['delete_side_wall'] == 2 OR $wall['delete_side_wall'] == 3 ) ) {
							if ( $sill_height ){
								//upper part
								$tiling_wall_height = $tiling_height;
								$wall_height = $op_height;
								$wall_y = ( $op_height - $sill_height ) / 2 + $sill_height;
								if ( $tiling_wall_height ) {
									if ( $tiling_wall_height > $wall_height ) {
										$tiling_wall_height = $wall_height;
									}
									if ( $tiling_wall_height > $sill_height ) {
										$wall_height = $op_height - $tiling_wall_height;
										$wall_y = ( $op_height - $tiling_wall_height ) / 2 + $tiling_wall_height;
										$return .= bimba_plane_entity_render( 'tiling', $room_name, ( $j . '-ri' ) , $wall_depth, ( $tiling_wall_height - $sill_height ),
												$wall_length / 2 - $op_offset_right , ( $tiling_wall_height - $sill_height ) / 2 + $sill_height, - $wall_depth / 2, '0', '-90', '0', $tiling_render );
									} else {
										$wall_height = $op_height - $sill_height;
									}
								}
								$return .= bimba_plane_entity_render( 'wall', $room_name, ( $j . '-ri' ) , $wall_depth , $wall_height ,
										$wall_length / 2 - $op_offset_right, $wall_y, - $wall_depth / 2, '0', '-90', '0', $wall_render );
								//lower part
								$tiling_wall_height = $tiling_height;
								$wall_height = $sill_height;
								$wall_y = $sill_height / 2;
								if ( $tiling_wall_height ) {
									if ( $tiling_wall_height > $wall_height ) {
										$tiling_wall_height = $wall_height;
									}
									$wall_height = $sill_height - $tiling_wall_height;
									$wall_y = ( $sill_height - $tiling_wall_height ) / 2 + $tiling_wall_height;
									$return .= bimba_plane_entity_render( 'tiling', $room_name, ( $j . '-ri2' ) , ( $wall_depth - $sill_depth ), $tiling_wall_height,
											$wall_length / 2 - $op_offset_right , $tiling_wall_height / 2 , - ( $wall_depth - $sill_depth ) / 2, '0', '-90', '0', $tiling_render );
								} elseif ( $sk_height ) {
									$wall_height = $sill_height - $sk_height;
									$wall_y = ( $sill_height - $sk_height ) / 2 + $sk_height;
									$return .= bimba_plane_entity_render_1_1( 'skirting', $room_name, ( $j . '-ri2' ) , ( $wall_depth - $sill_depth ), $sk_height,
											$wall_length / 2 - $op_offset_right , $sk_height / 2 , - ( $wall_depth - $sill_depth ) / 2, '0', '-90', '0', $room_skirting_render );
								}
								$return .= bimba_plane_entity_render( 'wall', $room_name, ( $j . '-ri2' ) , ( $wall_depth - $sill_depth ) , $wall_height ,
										$wall_length / 2 - $op_offset_right, $wall_y, - ( $wall_depth - $sill_depth ) / 2, '0', '-90', '0', $wall_render );
							} else {
								$tiling_wall_height = $tiling_height;
								$wall_height = $op_height;
								$wall_y = $op_height / 2;
								if ( $tiling_wall_height ) {
									if ( $tiling_wall_height > $wall_height ) {
										$tiling_wall_height = $wall_height;
									}
									$wall_height = $op_height - $tiling_wall_height;
									$wall_y = ( $op_height - $tiling_wall_height ) / 2 + $tiling_wall_height;
									$return .= bimba_plane_entity_render( 'tiling', $room_name, ( $j . '-ri' ) , $wall_depth, $tiling_wall_height,
											$wall_length / 2 - $op_offset_right , $tiling_wall_height / 2 , - $wall_depth / 2, '0', '-90', '0', $tiling_render );
								} elseif ( $sk_height ) {
									$wall_height = $op_height - $sk_height;
									$wall_y = ( $op_height - $sk_height ) / 2 + $sk_height;
									$return .= bimba_plane_entity_render_1_1( 'skirting', $room_name, ( $j . '-ri' ) , $wall_depth, $sk_height,
											$wall_length / 2 - $op_offset_right , $sk_height / 2 , - $wall_depth / 2, '0', '-90', '0', $room_skirting_render );
								}
								$return .= bimba_plane_entity_render( 'wall', $room_name, ( $j . '-ri' ) , $wall_depth , $wall_height ,
										$wall_length / 2 - $op_offset_right, $wall_y, - $wall_depth / 2, '0', '-90', '0', $wall_render );
							}
							
						}
						
						//davanzale
						if ( $sill_height AND $sill_depth ) {
							$return .= '
									<a-entity id="'.$room_name.'-dav-'.$j.'" 
									geometry="primitive: box; width: ' . $op_width . '; height: 0.03; depth: ' . ( $sill_depth + 0.03 ) . '"
									position="' . ( ( $op_offset + $op_width / 2 ) - $wall_length / 2 ) . ' ' . ( $sill_height - 0.015 ) . ' ' . ( - $wall_depth + $sill_depth / 2 ) . '" 
											material="color: #ffffff"></a-entity>';
						}
						
						//windowsill
						if ( $sill_height ) {
							$wall_height = $sill_height;
							$wall_y = $sill_height / 2;
							if ( $tiling_height ) {
								$tiling_sill_height = $tiling_height;
								if ( $tiling_sill_height > $sill_height ) {
									$tiling_sill_height = $sill_height;
								}
								$wall_height = $sill_height - $tiling_sill_height;
								$wall_y = ( $sill_height - $tiling_sill_height ) / 2 + $tiling_sill_height;
								$return .= bimba_plane_entity_render( 'tiling', $room_name, ( $j . '-sill' ) , $op_width , $tiling_sill_height,
										( $op_offset + $op_width / 2 ) - $wall_length / 2 , $tiling_sill_height / 2, - ( $wall_depth - $sill_depth ) ,
										'0', '0', '0', $tiling_render );
							} elseif ( $sk_height ) {
								$sk_sill_height = $sk_height;
								if ( $sk_sill_height > $sill_height ) {
									$sk_sill_height = $sill_height;
								}
								$wall_height = $sill_height - $sk_sill_height;
								$wall_y = ( $sill_height - $sk_sill_height ) / 2 + $sk_sill_height;
								$return .= bimba_plane_entity_render_1_1( 'skirting', $room_name, ( $j . '-sill' ) , $op_width , $sk_sill_height,
										( $op_offset + $op_width / 2 ) - $wall_length / 2 , $sk_sill_height / 2, - ( $wall_depth - $sill_depth ) , '0', '0', '0', $room_skirting_render );
							}
							$return .= bimba_plane_entity_render( 'wall', $room_name, ( $j . '-sill' ) , $op_width , $wall_height ,
									( $op_offset + $op_width / 2 ) - $wall_length / 2, $wall_y , - ( $wall_depth - $sill_depth ), '0', '0', '0', $wall_render );
						}
						//windowsill capping if side wall is deleted
						if ( $sill_height AND $sill_depth ) {
							$cap_height = $sill_height - 0.03;
							if ( $wall['delete_side_wall'] == 1 OR $wall['delete_side_wall'] == 3 ) {
								$wall_height = $cap_height ;
								$wall_y = ( $cap_height  ) / 2;
								if ( $tiling_height ) {
									$tiling_sill_height = $tiling_height;
									if ( $tiling_sill_height > $cap_height ) {
										$tiling_sill_height = $cap_height ;
									}
									$wall_height = $cap_height - $tiling_sill_height;
									$wall_y = ( $cap_height - $tiling_sill_height ) / 2 + $tiling_sill_height;
									$return .= bimba_plane_entity_render( 'tiling', $room_name, ( $j . '-sill-lcap' ) , $sill_depth , $tiling_sill_height,
											- $wall_length / 2 + $op_offset , $tiling_sill_height / 2, - ( $wall_depth - $sill_depth / 2 ) ,
											'0', '-90', '0', $tiling_render );
								} elseif ( $sk_height ) {
									$sk_sill_height = $sk_height;
									if ( $sk_sill_height > $cap_height ) {
										$sk_sill_height = $cap_height ;
									}
									$wall_height = $cap_height - $sk_sill_height;
									$wall_y = ( $cap_height - $sk_sill_height ) / 2 + $sk_sill_height;
									$return .= bimba_plane_entity_render_1_1( 'skirting', $room_name, ( $j . '-sill-lcap' ) , $sill_depth , $sk_sill_height,
											- $wall_length / 2 + $op_offset , $sk_sill_height / 2, - ( $wall_depth - $sill_depth / 2) , '0', '-90', '0', $room_skirting_render );
								}
								$return .= bimba_plane_entity_render( 'wall', $room_name, ( $j . '-sill-lcap' ) , $sill_depth , $wall_height ,
										- $wall_length / 2 + $op_offset , $wall_y , - ( $wall_depth - $sill_depth / 2), '0', '-90', '0', $wall_render );
							}
							if ( $wall['delete_side_wall'] == 2 OR $wall['delete_side_wall'] == 3 ) {
								$wall_height = $cap_height;
								$wall_y = $cap_height / 2;
								if ( $tiling_height ) {
									$tiling_sill_height = $tiling_height;
									if ( $tiling_sill_height > $cap_height ) {
										$tiling_sill_height = $cap_height;
									}
									$wall_height = $cap_height - $tiling_sill_height;
									$wall_y = ( $cap_height - $tiling_sill_height ) / 2 + $tiling_sill_height;
									$return .= bimba_plane_entity_render( 'tiling', $room_name, ( $j . '-sill-rcap' ) , $sill_depth , $tiling_sill_height,
											$wall_length / 2 - $op_offset_right , $tiling_sill_height / 2, - ( $wall_depth - $sill_depth / 2 ) ,
											'0', '90', '0', $tiling_render );
								} elseif ( $sk_height ) {
									$sk_sill_height = $sk_height;
									if ( $sk_sill_height > $cap_height ) {
										$sk_sill_height = $cap_height;
									}
									$wall_height = $cap_height - $sk_sill_height;
									$wall_y = ( $cap_height - $sk_sill_height ) / 2 + $sk_sill_height;
									$return .= bimba_plane_entity_render_1_1( 'skirting', $room_name, ( $j . '-sill-rcap' ) , $sill_depth , $sk_sill_height,
											$wall_length / 2 - $op_offset_right , $sk_sill_height / 2, - ( $wall_depth - $sill_depth / 2) , '0', '90', '0', $room_skirting_render );
								}
								$return .= bimba_plane_entity_render( 'wall', $room_name, ( $j . '-sill-rcap' ) , $sill_depth , $wall_height ,
										$wall_length / 2 - $op_offset_right , $wall_y , - ( $wall_depth - $sill_depth / 2), '0', '90', '0', $wall_render );
							}
						}
						//frame
						if ( $wall [ 'op_material' ] AND $op_width AND $op_height > $sill_height ) {
							
							$op_material = get_post_meta( $wall ['op_material'], '_3d_material_render', true );
							$op_render = 'src: #bimba-material-' . $wall [ 'op_material' ] . '; color: ' . $op_material [0] ['color'];
							$door_height = $op_height - $sill_height;
							$door_y = $door_height / 2 + $sill_height;
							
							switch ( $wall ['openings'] ){
								
								case 1://door
						
									$op_name = 'door';
									$frame_render = 'src: #bimba-frame-material-' . $wall [ 'op_material' ] . '; color: ' . $op_material [0] ['frame-color'];
									$return .='<a-entity id="' . $room_name . '-door-frame-ent-' . $j . '" position="' . ( - $wall_length / 2 + $op_offset ) . ' 0 ' . ( -$wall_depth ) . '">';
									$return .= bimba_3d_ambient_render_door_frame($room_name, $j, $door_height, $door_y, $frame_render, $op_width );
									$return .='</a-entity>';
						
								break;
										
								case 2://window
										
									$op_name = 'window';
									
								break;
							}//end switch opening
							
							switch ($wall ['animation']) {//door animation
									
								case 0://no movement
									$return .='<a-entity id="' . $room_name . '-' . $op_name . '-ent-' . $j . '" position="' . ( - $wall_length / 2 + $op_offset ) . ' 0 ' . ( -$wall_depth ) . '">';
									$return .='<a-entity id="'.$room_name.'-' . $op_name . '-'.$j.'" geometry="primitive: box; width: '.$op_width.'; height: '.$door_height.'; depth: 0.05"
											position="' . ( $op_width/2 ) . ' ' . $door_y . ' -0.025" material="' . $op_render . '"></a-entity></a-entity>';
									break;
							
								case 1://left hinge
									$return .='<a-entity id="' . $room_name . '-' . $op_name . '-ent-' . $j . '" position="' . ( - $wall_length / 2 + $op_offset ) . ' 0 ' . ( -$wall_depth ) . '">
										<a-animation attribute="rotation" from="0 0 0" to="0 -90 0" begin="click" repeat="1" direction="alternate"></a-animation>
										<a-entity id="'.$room_name.'-' . $op_name . '-'.$j.'" geometry="primitive: box; width: '.$op_width.'; height: '.$door_height.'; depth: 0.05"
											position="' . ( $op_width/2 ) . ' ' . $door_y . ' -0.025" material="' . $op_render . '"></a-entity></a-entity>';
									break;
							
								case 2://right hinge
									$return .='<a-entity id="' . $room_name . '-' . $op_name . '-ent-' . $j . '" position="' . ( -$wall_length / 2 + $op_offset + $op_width ) . ' 0 ' . ( -$wall_depth ) . '">
										<a-animation attribute="rotation" from="0 0 0" to="0 90 0" begin="click" repeat="1" direction="alternate"></a-animation>
										<a-entity id="'.$room_name.'-' . $op_name . '-'.$j.'" geometry="primitive: box; width: '. $op_width .'; height: '. $door_height .'; depth: 0.05"
											position="' . ( - $op_width/2 ) . ' ' . $door_y . ' -0.025" material="' . $op_render . '"></a-entity></a-entity>';
									break;
										
								case 5://double hinges
									$return .='<a-entity id="' . $room_name . '-' . $op_name . '-ent-' . $j . '" position="' . ( - $wall_length / 2 + $op_offset ) . ' 0 ' . ( -$wall_depth ) . '">
													<a-entity id="' . $room_name . '-' . $op_name . '-l-ent-' . $j . '" position="0 0 0">
										<a-animation attribute="rotation" from="0 0 0" to="0 -90 0" begin="click" repeat="1" direction="alternate"></a-animation>
										<a-entity id="'.$room_name.'-' . $op_name . '-l-'.$j.'" geometry="primitive: box; width: '. ($op_width / 2) .'; height: '.$door_height.'; depth: 0.05"
											position="' . ( $op_width/4 ) . ' ' . $door_y . ' -0.025" material="' . $op_render . '"></a-entity></a-entity>
													<a-entity id="' . $room_name . '-' . $op_name . '-r-ent-' . $j . '" position="' . $op_width . ' 0 0">
										<a-animation attribute="rotation" from="0 0 0" to="0 90 0" begin="click" repeat="1" direction="alternate"></a-animation>
										<a-entity id="'.$room_name.'-' . $op_name . '-r-'.$j.'" geometry="primitive: box; width: '. ($op_width / 2) .'; height: '. $door_height .'; depth: 0.05"
											position="' . ( - $op_width/4 ) . ' ' . $door_y . ' -0.025" material="' . $op_render . '"></a-entity></a-entity></a-entity>';
									break;
							
								case 3://slides left
									$return .='<a-entity id="' . $room_name . '-' . $op_name . '-ent-' . $j . '" position="' . ( - $wall_length / 2 + $op_offset ) . ' 0 ' . ( -$wall_depth ) . '">
										<a-animation attribute="position" from="' . ( - $wall_length / 2 + $op_offset ) . ' 0 ' . ( -$wall_depth ) . '"
												to="' . ( - $wall_length / 2 + $op_offset - $op_width ) . ' 0 ' . ( -$wall_depth ) . '" begin="click" repeat="1" direction="alternate"></a-animation>
										<a-entity id="'.$room_name.'-' . $op_name . '-'.$j.'" geometry="primitive: box; width: '.$op_width.'; height: '.$door_height.'; depth: 0.05"
											position="' . ( $op_width/2 ) . ' ' . $door_y . ' -0.05" material="' . $op_render . '"></a-entity></a-entity>';
									break;
							
								case 4://slides right
									$return .='<a-entity id="' . $room_name . '-' . $op_name . '-ent-' . $j . '" position="' . ( - $wall_length / 2 + $op_offset ) . ' 0 ' . ( -$wall_depth ) . '">
										<a-animation attribute="position" from="' . ( - $wall_length / 2 + $op_offset ) . ' 0 ' . ( -$wall_depth ) . '"
												to="' . ( - $wall_length / 2 + $op_offset + $op_width ) . ' 0 ' . ( -$wall_depth ) . '" begin="click" repeat="1" direction="alternate"></a-animation>
										<a-entity id="'.$room_name.'-' . $op_name . '-'.$j.'" geometry="primitive: box; width: '.$op_width.'; height: '.$door_height.'; depth: 0.05"
											position="' . ( $op_width/2 ) . ' ' . $door_y . ' -0.05" material="' . $op_render . '"></a-entity></a-entity>';
									break;
										
								case 6://double slides
									$return .='<a-entity id="' . $room_name . '-' . $op_name . '-ent-' . $j . '" position="' . ( - $wall_length / 2 + $op_offset ) . ' 0 ' . ( -$wall_depth ) . '">
												<a-entity id="' . $room_name . '-' . $op_name . '-l-ent-' . $j . '" position="0 0 0">
												<a-animation attribute="position" from="0 0 0" to="'. -($op_width / 2) .' 0 0" begin="click" repeat="1" direction="alternate"></a-animation>
												<a-entity id="'.$room_name.'-' . $op_name . '-l-'.$j.'" geometry="primitive: box; width: '. ($op_width / 2) .'; height: '.$door_height.'; depth: 0.05"
												position="' . ( $op_width/4 ) . ' ' . $door_y . ' -0.05" material="' . $op_render . '"></a-entity></a-entity>
												<a-entity id="' . $room_name . '-' . $op_name . '-r-ent-' . $j . '" position="' . $op_width . ' 0 0">
												<a-animation attribute="position" from="'. $op_width .' 0 0" to="'. ($op_width * 1.5) .' 0 0" begin="click" repeat="1" direction="alternate"></a-animation>
												<a-entity id="'.$room_name.'-' . $op_name . '-r-'.$j.'" geometry="primitive: box; width: '. ($op_width / 2) .'; height: '. $door_height .'; depth: 0.05"
												position="' . ( - $op_width/4 ) . ' ' . $door_y . ' -0.05" material="' . $op_render . '"></a-entity></a-entity></a-entity>';
									break;
							}//end switch opening animation
								
						}//end if opening material
						//floor
						if ( $wall_depth ) {
							$return .= bimba_plane_entity_render( 'floor', $room_name, $j , $op_width , ( $wall_depth - $sill_depth ),
									( $op_offset + $op_width / 2 ) - $wall_length / 2, '0' , - ( $wall_depth - $sill_depth ) / 2, '-90', '0', '0',
									$room_floor_render );
						}
						
					break;
				}//end if opening
				
			}//end if no wall image
			
			//now objects
			$ii = 0;
			foreach ($objects as $object) {
				$jj = $ii + 1;
				if ( $object [ 'wall-id' ] == $j ) {
					$object_offset = $object ['offset'] / 100;
					switch ( $object [ 'type' ] ) {
						
						case 0://none
						break;
						
						case 1://bed
							$object_width = 0.8/2;
							if ( $object_offset == 0 ) {
								$object_offset = ( $wall_length ) / 2 - $object_width;
							} elseif ( $object_offset < 0 ) {
								$object_offset = 0;
							}
							$return .='
									<a-entity id="' . $room_name . '-bed-' . $jj . '" position="' . ( - $wall_length / 2 + $object_offset + $object_width) . ' 0 ' . $object ['distance'] / 100 . '">';
							$return .='<a-entity geometry="primitive: box; width: 0.05; height: 0.1; depth: 0.05" position="-0.375 0.05 0.025" material="color: white;"></a-entity>';
							$return .='<a-entity geometry="primitive: box; width: 0.05; height: 0.1; depth: 0.05" position="0.375 0.05 0.025" material="color: white;"></a-entity>';
							$return .='<a-entity geometry="primitive: box; width: 0.05; height: 0.1; depth: 0.05" position="-0.375 0.05 1.875" material="color: white;"></a-entity>';
							$return .='<a-entity geometry="primitive: box; width: 0.05; height: 0.1; depth: 0.05" position="0.375 0.05 1.875" material="color: white;"></a-entity>';
							$return .='<a-entity geometry="primitive: box; width: 0.8; height: 0.35; depth: 1.9" position="0 0.275 0.95" material="color: red;"></a-entity>';
							$return .='<a-entity geometry="primitive: box; width: 0.7; height: 0.1; depth: 0.35" position="0 0.5 0.175" material="color: white;"></a-entity>';
							$return .= '
									</a-entity>';
						break;
							
						case 2://chair and table
							$object_width = 1.5/2;
							if ( $object_offset == 0 ) {
								$object_offset = ( $wall_length ) / 2 - $object_width;
							} elseif ( $object_offset < 0 ) {
								$object_offset = 0;
							}
							$return .='
									<a-entity id="' . $room_name . '-table-' . $jj . '" position="' . ( - $wall_length / 2 + $object_offset + $object_width) . ' 0 ' . $object ['distance'] / 100 . '">';
							$return .='<a-entity geometry="primitive: box; width: 0.05; height: 0.7; depth: 0.05" position="-0.725 0.35 0.025" material="color: white;"></a-entity>';
							$return .='<a-entity geometry="primitive: box; width: 0.05; height: 0.7; depth: 0.05" position="0.725 0.35 0.025" material="color: white;"></a-entity>';
							$return .='<a-entity geometry="primitive: box; width: 0.05; height: 0.7; depth: 0.05" position="-0.725 0.35 0.725" material="color: white;"></a-entity>';
							$return .='<a-entity geometry="primitive: box; width: 0.05; height: 0.7; depth: 0.05" position="0.725 0.35 0.725" material="color: white;"></a-entity>';
							$return .='<a-entity geometry="primitive: box; width: 1.5; height: 0.05; depth: 0.75" position="0 0.725 0.375" material="color: white;"></a-entity>';
							$return .= '<a-entity id="' . $room_name . '-chair-' . $jj . '" position="0 0 0.75" rotation="0 0 0">';
							$return .= bimba_3d_ambient_render_chair();	
							$return .= '</a-entity>';
							
							$return .= '
									</a-entity>';
						break;
								
						case 3://module
							$object_module = $object ['module_width'] / 100;
							if ( ! $object_module) {
								$object_module = 0.6;
							}
							$object_height = $object ['height'] / 100;
							if ( ! $object_height OR $object_height > $height ) {
								$object_height = $height;
							}
							$object_depth = $object ['depth'] / 100;
							if ( ! $object_depth) {
								$object_depth = 0.6;
							}
							$object_altitude = $object ['altitude'] / 100;
							if ( $object_altitude + $object_height > $height ) {
								$object_altitude = $height - $object_height;
							}
							if ( $object ['width'] ) {
								$object_width = intval( $object ['width'] / 100 / $object_module ) * $object_module;
							} else {
								$object_width = intval( ( $wall_length - $object_offset ) / $object_module ) * $object_module;
							}
							
							if ( $object_width > $wall_length ) {
								$object_width = intval( $wall_length / $object_module ) * $object_module;
							}
							if ( ( $object_width + $object_offset ) > $wall_length ) {
								$object_offset = $wall_length - $object_width;
							}
							
							if ( $object_offset == 0 ) {
								$object_offset = ( $wall_length - $object_width ) / 2;
							} elseif ( $object_offset < 0 ) {
								$object_offset = 0;
							}
							$return .='
									<a-entity id="' . $room_name . '-module-' . $jj . '" position="' . ( - $wall_length / 2 + $object_offset ) . ' 0 ' . $object ['distance'] / 100 . '">';
							for ($k = 0; $k < ( $object_width / $object_module ); $k++) {
								$return .='<a-entity geometry="primitive: box; width: ' . ( $object_module - 0.01 ) . '; height: ' . $object_height . '; depth: ' . $object_depth . '" 
										position="' . ( $object_module / 2 + $object_module * $k ) . ' ' . ( $object_height / 2  + $object_altitude ) . ' ' . $object_depth / 2 . '" material="color: ' . $object ['color'] . ';"></a-entity>';
							}
							$return .= '
									</a-entity>';
						break;
						
						case 4://sideboard
							$object_width = 1.8/2;
							if ( $object_offset == 0 ) {
								$object_offset = ( $wall_length ) / 2 - $object_width;
							} elseif ( $object_offset < 0 ) {
								$object_offset = 0;
							}
							$return .='
									<a-entity id="' . $room_name . '-sideboard-' . $jj . '" position="' . ( - $wall_length / 2 + $object_offset + $object_width) . ' 0 ' . $object ['distance'] / 100 . '">';
							$return .='<a-entity geometry="primitive: box; width: 1.8; height: 0.52; depth: 0.52" position="0 0.46 0.26" material="color: #8b877b;"></a-entity>';
							$return .= '<a-entity id="' . $room_name . '-sideboard-' . $jj . '-label" scale="2 2 2" position="0 0.46 0.53" text="value: Bontempi Aly Glass; align: center;"></a-entity>';
							$return .= '
									</a-entity>';
							break;
						
						case 5://cupboard
							$object_width = 1.1/2;
							if ( $object_offset == 0 ) {
								$object_offset = ( $wall_length ) / 2 - $object_width;
							} elseif ( $object_offset < 0 ) {
								$object_offset = 0;
							}
							$return .='
									<a-entity id="' . $room_name . '-cupboard-' . $jj . '" position="' . ( - $wall_length / 2 + $object_offset + $object_width) . ' 0 ' . $object ['distance'] / 100 . '">';
							$return .='<a-entity geometry="primitive: box; width: 1.1; height: 1.8; depth: 0.4" position="0 0.9 0.2" material="color: #ad7c44;"></a-entity>';
							$return .= '
									</a-entity>';
							break;
							
						case 6://center table with 4 chairs
							$object_width = 0.9/2;
							if ( $object_offset == 0 ) {
								$object_offset = ( $wall_length ) / 2 - $object_width;
							} elseif ( $object_offset < 0 ) {
								$object_offset = 0;
							}
							$return .='
									<a-entity id="' . $room_name . '-center-table-' . $jj . '" position="' . ( - $wall_length / 2 + $object_offset + $object_width) . ' 0 ' . $object ['distance'] / 100 . '">';
							$return .='<a-entity geometry="primitive: box; width: 0.05; height: 0.7; depth: 0.02" position="-0.425 0.35 0.025" rotation="0 -45 0" material="color: white;"></a-entity>';//table legs
							$return .='<a-entity geometry="primitive: box; width: 0.05; height: 0.7; depth: 0.02" position="0.425 0.35 0.025" rotation="0 45 0" material="color: white;"></a-entity>';
							$return .='<a-entity geometry="primitive: box; width: 0.05; height: 0.7; depth: 0.02" position="-0.425 0.35 1.375" rotation="0 45 0" material="color: white;"></a-entity>';
							$return .='<a-entity geometry="primitive: box; width: 0.05; height: 0.7; depth: 0.02" position="0.425 0.35 1.375" rotation="0 -45 0" material="color: white;"></a-entity>';
							$return .='<a-entity geometry="primitive: box; width: 0.9; height: 0.05; depth: 1.4" position="0 0.725 0.7" material="color: #8b877b;"></a-entity>';//table plane
							$return .= '
									<a-entity id="' . $room_name . '-chair-' . $jj . '-a" position="0 0 0" rotation="0 180 0">';
							$return .= bimba_3d_ambient_render_chair();	
							$return .= '
									</a-entity>';
							$return .= '
									<a-entity id="' . $room_name . '-chair-' . $jj . '-b" position="0 0 1.4" rotation="0 0 0">';
							$return .= bimba_3d_ambient_render_chair();
							$return .= '
									</a-entity>';
							$return .= '
									<a-entity id="' . $room_name . '-chair-' . $jj . '-c" position="-0.45 0 0.7" rotation="0 270 0">';
							$return .= bimba_3d_ambient_render_chair();
							$return .= '
									</a-entity>';
							$return .= '
									<a-entity id="' . $room_name . '-chair-' . $jj . '-d" position="0.45 0 0.7" rotation="0 90 0">';
							$return .= bimba_3d_ambient_render_chair();
							$return .= '
									</a-entity>';
							$return .= '
									</a-entity>';
							break;
							
						//case 7://architect look-at component not 
							//$return .= '
									//<a-entity id="' . $room_name . '-architect-ent-' . $jj . '" position="' . $object_offset . ' 0 ' . $object ['distance'] / 100 . '" look-at="#camera-foot">
									//<a-entity id="' . $room_name . '-architect-' . $jj . '" geometry="primitive: plane; width: 0.56; height: 1.70;" position="0 0.85 0" material="src: #bimba-architect">
									//</a-entity></a-entity>';
						//break;
						
						case 7://note
							$return .= '
							<a-entity id="' . $room_name . '-note-ent-' . $jj . '" position="' . ( - $wall_length / 2 + $object_offset + 0.3) . ' ' . $object ['altitude'] / 100 . ' ' . $object ['distance'] / 100 . '">
							<a-entity id="' . $room_name . '-note-' . $jj . '" scale="2 2 2" geometry="primitive: plane; width: 0.3; height: auto;" position="0 0 0.01" material="color: #e7e662; side: double;" 
							text="value: ' . $object ['text'] . '; color: black; align: center; width: 0.3;">
							</a-entity></a-entity>';
						break;
						
					}					
										
				}
				$ii ++;
			}
			
			$return .='
					</a-entity><!-- End '.$room_name.' Wall-'.$j.' Entity -->';//end of wall entity
		}//end if wall length
		$i ++;
	}
	
	$return .= '
			</a-entity><!-- End '.$room_name.' Room Entity -->';//end of room entity
	
	$args = array(
		'room_height'		=> $height,
		'floor_area'		=> $floor_area,
		'volume'			=> $height * $floor_area,
		'floor_area_op'		=> $floor_area_op,
		'room_paint_surf'	=> $paint_surf,
		'room_tiling_surf'	=> $tiling_surf,
		'room_skirt_length'	=> $sk_length,
		'door_number'		=> $door_number,
		'door_surf'		=> $door_surf,
		'window_number'		=> $window_number,
		'window_surf'		=> $window_surf,
	);
	
	update_post_meta( $id, '_3d_ambient_data', $args );
	
	update_post_meta( $id, '_3d_ambient_general_settings', $room_set );//this is feedback, if settings were changed
	update_post_meta( $id, '_3d_ambient_group_walls', $walls );
	
	return $return;
}

function bimba_3d_ambient_render_door_frame($room_name, $j, $door_height, $door_y, $frame_render, $op_width ) {
	
	$return = '<a-entity id="' . $room_name . '-door-l-frame-' . $j . '" 
				geometry="primitive: box; width: 0.08; height: '.$door_height.'; depth: 0.12"
				position="-0.03 ' . $door_y . ' -0.05" material="' . $frame_render . '"></a-entity>
				<a-entity id="' . $room_name . '-door-r-frame-' . $j . '" 
				geometry="primitive: box; width: 0.08; height: '.$door_height.'; depth: 0.12"
				position="'. ( $op_width + 0.03 ) . ' ' . $door_y . ' -0.05" material="' . $frame_render . '"></a-entity>
				<a-entity id="' . $room_name . '-door-u-frame-' . $j . '" 
				geometry="primitive: box; width: 0.08 ; height: ' . ( $op_width + 0.14 ) . '; depth: 0.12"
				position="'. ( $op_width / 2 ) . ' ' . ( $door_height + 0.04) . ' -0.05" rotation="0 0 -90" material="' . $frame_render . '"></a-entity>';
	
	return $return;										
}

/**
 * This function sets the room entity parameters
 */

function bimba_start_room_entity( $id, $room_name, $parent_coord, $global_flag){
	
	if ( $global_flag ) {//if global flag exists you have to consider global positioning
	
		$child_diff = get_post_meta( $id, '_3d_ambient_global_diff', true );
		$child_rot = ( $child_diff[0]['rot'] - $parent_coord [ 'parent_rot' ] ) * 90;
		$child_y = ( $child_diff[0]['y'] - $parent_coord [ 'parent_y' ] ) / 100;
	
		switch ( $parent_coord [ 'parent_rot' ] ){//transform coordinates depend on parent rotation
			case 0:
				$child_x = ( $child_diff[0]['x'] - $parent_coord [ 'parent_x' ] ) / 100;
				$child_z = ( $child_diff[0]['z'] - $parent_coord [ 'parent_z' ] ) / 100;
				break;
					
			case 1:
				$child_x = ( $child_diff[0]['z'] - $parent_coord [ 'parent_z' ] ) / 100;
				$child_z = ( $child_diff[0]['x'] - $parent_coord [ 'parent_x' ] ) / 100;
				break;
	
			case 2:
				$child_x = - ( $child_diff[0]['x'] - $parent_coord [ 'parent_x' ] ) / 100;
				$child_z = - ( $child_diff[0]['z'] - $parent_coord [ 'parent_z' ] ) / 100;
				break;
					
			case 3:
				$child_x = - ( $child_diff[0]['z'] - $parent_coord [ 'parent_z' ] ) / 100;
				$child_z = - ( $child_diff[0]['x'] - $parent_coord [ 'parent_x' ] ) / 100;
				break;
		}
		$return .= '
				<a-entity id="' . $room_name . '-room" position="' . $child_x . ' ' . $child_y . ' ' . $child_z . '" rotation="0 ' . $child_rot . ' 0">';
	} else {
		$return .= '
				<a-entity id="' . $room_name . '-room">';
	}
	$return .= '
			<a-entity id="' . $room_name . '-label" scale="2 2 2" side="double" position="0 0.1 0" text="value: ' . get_the_title($id) . ';color: black; align: center; side: double;"></a-entity>';
	
	return $return;
}

function bimba_3d_ambient_render_by_material( $material_id ) {
	$material_array = get_post_meta( $material_id, '_3d_material_render', true );
	if ( $material_array ) {
		$material_render = 'src: #bimba-material-' . $material_id . '; color:' . $material_array [0] ['color'];
	} else {
		$material_render = 'color: #ffffff';
	}
	return $material_render;
}

function bimba_3d_ambient_render_chair(){
	$return = '
			<a-entity position="0 0 -0.45">';
	$return .='<a-entity geometry="primitive: cylinder; radius: 0.015; height: 0.4;" position="-0.2 0.2 0.025" material="color: white;"></a-entity>';
	$return .='<a-entity geometry="primitive: cylinder; radius: 0.015; height: 0.4;" position="0.2 0.2 0.025" material="color: white;"></a-entity>';
	$return .='<a-entity geometry="primitive: cylinder; radius: 0.015; height: 0.75;" position="-0.2 0.375 0.475" material="color: white;"></a-entity>';
	$return .='<a-entity geometry="primitive: cylinder; radius: 0.015; height: 0.75;" position="0.2 0.375 0.475" material="color: white;"></a-entity>';
	$return .='<a-entity geometry="primitive: box; width: 0.45; height: 0.05; depth: 0.45" position="0 0.425 0.225" material="color: #8b877b;"></a-entity>';
	$return .='<a-entity geometry="primitive: box; width: 0.45; height: 0.25; depth: 0.03" position="0 0.875 0.475" material="color: #8b877b;"></a-entity>';
	$return .='<a-animation attribute="position" from="0 0 -0.45" to="0 0 0" begin="click" repeat="1" direction="alternate"></a-animation>';
	$return .='
			</a-entity>';
	return $return;
}

/**
 * Display 3d ambient data
 */

function bimba_3d_ambient_display_data( $id ){
	
	global $room_warnings;
	global $wall_warnings;
	
	$data = get_post_meta( $id, '_3d_ambient_data', true );
	
	$display_data = array(
		array( 'string' => esc_html__('Room area is %s m2', 'bim-ba' ), 'data' => $data [ 'floor_area' ] ),
		array( 'string' => esc_html__('Room height is %s m', 'bim-ba' ), 'data' => $data [ 'room_height' ] ),
		array( 'string' => esc_html__('Room volume is %s m3', 'bim-ba' ), 'data' => $data [ 'volume' ] ),
		array( 'string' => esc_html__('Floor area is %s m2', 'bim-ba' ), 'data' => ( $data [ 'floor_area' ] + $data [ 'floor_area_op' ] ) ),
		array( 'string' => esc_html__('Room wall surface is %s m2', 'bim-ba' ), 'data' => ( $data [ 'room_paint_surf' ] - $data [ 'floor_area' ] + $data [ 'room_tiling_surf' ] ) ),
		array( 'string' => esc_html__('Room painted surface is %s m2', 'bim-ba' ), 'data' => $data [ 'room_paint_surf' ] ),
		array( 'string' => esc_html__('Room tiling surface is %s m2', 'bim-ba' ), 'data' => $data [ 'room_tiling_surf' ] ),
		array( 'string' => esc_html__('Room skirting length is %s m', 'bim-ba' ), 'data' => $data [ 'room_skirt_length' ] ),
		array( 'string' => esc_html( _n('Room has %s door', 'Room has %s doors', $data [ 'door_number' ], 'bim-ba' ) ), 'data' => $data [ 'door_number' ] ),
		array( 'string' => esc_html__('Door surface is %s m2', 'bim-ba' ), 'data' => $data [ 'door_surf' ] ),
		array( 'string' => esc_html( _n('Room has %s window', 'Room has %s windows', $data [ 'window_number' ], 'bim-ba' ) ), 'data' => $data [ 'window_number' ] ),
		array( 'string' => esc_html__('Window surface is %s m2', 'bim-ba' ), 'data' => $data [ 'window_surf' ] ),
	);
	
	$return .= '<h5>' . sprintf( esc_html__('Data for room "%s"', 'bim-ba' ), get_the_title($id) ) . '</h5>';
	//warnings
	if ( ! $data [ 'room_height' ] ) {
		$return .= '<p>' . esc_html__('Room height is null, so it was not rendered.', 'bim-ba' ) . '</p>';
	} else {
		foreach ( $wall_warnings[ $id ] as $wall_warning ) {
			$return .= '<p>' . sprintf( esc_html__('Wall %s has no length, so it was not rendered.', 'bim-ba' ), $wall_warning ) . '</p>';
		}
		
		if ( $room_warnings [ $id ]['x'] OR $room_warnings [ $id ]['z'] ) {
			$return .= '<p>' . sprintf( esc_html__('There is a gap in the room polygon, x=%s cm, z=%s cm.', 'bim-ba' ), round( $room_warnings [ $id ]['x'] ), round( $room_warnings [ $id ]['z'] ) ) . '</p>';
		}
		
		if ( $data [ 'floor_area' ] + $data [ 'floor_area_op' ] < 9 ) {
			$return .= '<p>' . esc_html__('WARNING! Room surface is less than 9 m2', 'bim-ba' ) . '</p>';
		}
		if ( $data [ 'window_surf' ] AND ( $data [ 'floor_area' ] + $data [ 'floor_area_op' ] ) / $data [ 'window_surf' ] > 8 ) {
			$return .= '<p>' . esc_html__('WARNING! Window surface is less than 1/8th of floor area', 'bim-ba' ) . '</p>';
		}
		
		$return .= '<ul>';
		foreach ($display_data as $display) {
			if ( $display [ 'data' ] ) {
				$return .= '<li>' . sprintf( $display [ 'string' ], round( $display [ 'data' ], 2 ) ) . '</li>';
			}
		}
		$return .= '</ul>';
		
	}
	
	return $return;
}

function bimba_3d_ambient_display_total_data( $total_data ){
	$display_data = array(
			array( 'string' => esc_html__('Total area is %s m2', 'bim-ba' ), 'data' => $total_data [ 'floor_area' ] ),
			array( 'string' => esc_html__('Total volume is %s m3', 'bim-ba' ), 'data' => $total_data [ 'volume' ] ),
			array( 'string' => esc_html__('Total floor area is %s m2', 'bim-ba' ), 'data' => ( $total_data [ 'floor_area' ] + $total_data [ 'floor_area_op' ] ) ),
			array( 'string' => esc_html__('Total wall surface is %s m2', 'bim-ba' ), 'data' => ( $total_data [ 'room_paint_surf' ] - $total_data [ 'floor_area' ] + $total_data [ 'room_tiling_surf' ] ) ),
			array( 'string' => esc_html__('Total painted surface is %s m2', 'bim-ba' ), 'data' => $total_data [ 'room_paint_surf' ] ),
			array( 'string' => esc_html__('Total tiling surface is %s m2', 'bim-ba' ), 'data' => $total_data [ 'room_tiling_surf' ] ),
			array( 'string' => esc_html__('Total skirting length is %s m', 'bim-ba' ), 'data' => $total_data [ 'room_skirt_length' ] ),
			array( 'string' => esc_html( _n('In total there is %s door', 'In total there are %s doors', $total_data [ 'door_number' ], 'bim-ba' ) ), 'data' => $total_data [ 'door_number' ] ),
			array( 'string' => esc_html__('Total door surface is %s m2', 'bim-ba' ), 'data' => $total_data [ 'door_surf' ] ),
			array( 'string' => esc_html( _n('In total there is %s window', 'In total there are %s windows', $total_data [ 'window_number' ], 'bim-ba' ) ), 'data' => $total_data [ 'window_number' ] ),
			array( 'string' => esc_html__('Total window surface is %s m2', 'bim-ba' ), 'data' => $total_data [ 'window_surf' ] ),
	);
	$return .= '<h5>' . esc_html__('Total Data', 'bim-ba' ) . '</h5><ul>';
	
	foreach ($display_data as $display) {
		if ( $display [ 'data' ] ) {
			$return .= '<li>' . sprintf( $display [ 'string' ], round( $display [ 'data' ], 2 ) ) . '</li>';
		}
	}
	$return .= '</ul>';
	
	return $return;
}

function bimba_plane_entity_render( $type, $room_name, $i, $width, $height, $px, $py, $pz, $rx, $ry, $rz, $render ){
	$return = '
			<a-entity id="' . $room_name . '-' . $type . '-' . $i . '" 
				geometry="primitive: plane; width: ' . $width . '; height: ' . $height . '" 
				position="' . $px . ' ' . $py . ' ' . $pz . '" 
				rotation="' . $rx . ' ' . $ry . ' ' . $rz . '" 
				material="' . $render . '; repeat: ' . $width . ' ' . $height . '">
			</a-entity>';
	return $return;
}

function bimba_plane_entity_render_1_1( $type, $room_name, $i, $width, $height, $px, $py, $pz, $rx, $ry, $rz, $render ){
	$return = '
			<a-entity id="' . $room_name . '-' . $type . '-' . $i . '"
				geometry="primitive: plane; width: ' . $width . '; height: ' . $height . '"
				position="' . $px . ' ' . $py . ' ' . $pz . '"
				rotation="' . $rx . ' ' . $ry . ' ' . $rz . '"
				material="' . $render . '; repeat: 1 1 ">
			</a-entity>';
	return $return;
}

function bimba_sum_arrays($array1, $array2) {
	$array = array();
	foreach($array1 as $index => $value) {
		$array[$index] = isset($array2[$index]) ? $array2[$index] + $value : $value;
	}
	return $array;
}

/*
 * TODO Function creates post duplicate as a draft and redirects then to the edit post screen
 */
function rd_duplicate_post_as_draft(){
	global $wpdb;
	if (! ( isset( $_GET['post']) || isset( $_POST['post'])  || ( isset($_REQUEST['action']) && 'rd_duplicate_post_as_draft' == $_REQUEST['action'] ) ) ) {
		wp_die('No post to duplicate has been supplied!');
	}
 
	/*
	 * get the original post id
	 */
	$post_id = (isset($_GET['post']) ? absint( $_GET['post'] ) : absint( $_POST['post'] ) );
	/*
	 * and all the original post data then
	 */
	$post = get_post( $post_id );
 
	/*
	 * if you don't want current user to be the new post author,
	 * then change next couple of lines to this: $new_post_author = $post->post_author;
	 */
	$current_user = wp_get_current_user();
	$new_post_author = $current_user->ID;
 
	/*
	 * if post data exists, create the post duplicate
	 */
	if (isset( $post ) && $post != null) {
 
		/*
		 * new post data array
		 */
		$args = array(
			'comment_status' => $post->comment_status,
			'ping_status'    => $post->ping_status,
			'post_author'    => $new_post_author,
			'post_content'   => $post->post_content,
			'post_excerpt'   => $post->post_excerpt,
			'post_name'      => $post->post_name,
			'post_parent'    => $post->post_parent,
			'post_password'  => $post->post_password,
			'post_status'    => 'draft',
			'post_title'     => $post->post_title,
			'post_type'      => $post->post_type,
			'to_ping'        => $post->to_ping,
			'menu_order'     => $post->menu_order
		);
 
		/*
		 * insert the post by wp_insert_post() function
		 */
		$new_post_id = wp_insert_post( $args );
 
		/*
		 * get all current post terms ad set them to the new post draft
		 */
		$taxonomies = get_object_taxonomies($post->post_type); // returns array of taxonomy names for post type, ex array("category", "post_tag");
		foreach ($taxonomies as $taxonomy) {
			$post_terms = wp_get_object_terms($post_id, $taxonomy, array('fields' => 'slugs'));
			wp_set_object_terms($new_post_id, $post_terms, $taxonomy, false);
		}
 
		/*
		 * duplicate all post meta just in two SQL queries
		 */
		$post_meta_infos = $wpdb->get_results("SELECT meta_key, meta_value FROM $wpdb->postmeta WHERE post_id=$post_id");
		if (count($post_meta_infos)!=0) {
			$sql_query = "INSERT INTO $wpdb->postmeta (post_id, meta_key, meta_value) ";
			foreach ($post_meta_infos as $meta_info) {
				$meta_key = $meta_info->meta_key;
				$meta_value = addslashes($meta_info->meta_value);
				$sql_query_sel[]= "SELECT $new_post_id, '$meta_key', '$meta_value'";
			}
			$sql_query.= implode(" UNION ALL ", $sql_query_sel);
			$wpdb->query($sql_query);
		}
 
 
		/*
		 * finally, redirect to the edit post screen for the new draft
		 */
		wp_redirect( admin_url( 'post.php?action=edit&post=' . $new_post_id ) );
		exit;
	} else {
		wp_die('Post creation failed, could not find original post: ' . $post_id);
	}
}
add_action( 'admin_action_rd_duplicate_post_as_draft', 'rd_duplicate_post_as_draft' );
 
/*
 * Add the duplicate link to action list for post_row_actions
 */
function rd_duplicate_post_link( $actions, $post ) {
	if (current_user_can('edit_posts')) {
		$actions['duplicate'] = '<a href="admin.php?action=rd_duplicate_post_as_draft&amp;post=' . $post->ID . '" title="Duplicate this item" rel="permalink">Duplicate</a>';
	}
	return $actions;
}
 
//add_filter( 'post_row_actions', 'rd_duplicate_post_link', 10, 2 );